@@ -107,29 +107,25 @@ def print_header(procs_status, num_procs):
         dashes, empty_dashes = get_dashes(perc)
         print_line(" CPU%-2s [%s%s] %5s%%" % (cpu_num, dashes, empty_dashes,
                                               perc))
-    # physmem usage (on linux we include buffers and cached values
-    # to match htop results)
-    phymem = psutil.phymem_usage()
-    dashes, empty_dashes = get_dashes(phymem.percent)
-    buffers = getattr(psutil, 'phymem_buffers', lambda: 0)()
-    cached = getattr(psutil, 'cached_phymem', lambda: 0)()
-    used = phymem.total - (phymem.free + buffers + cached)
+    mem = psutil.virtual_memory()
+    dashes, empty_dashes = get_dashes(mem.percent)
+    used = mem.total - mem.available
     line = " Mem   [%s%s] %5s%% %6s/%s" % (
         dashes, empty_dashes,
-        phymem.percent,
+        mem.percent,
         str(int(used / 1024 / 1024)) + "M",
-        str(int(phymem.total / 1024 / 1024)) + "M"
+        str(int(mem.total / 1024 / 1024)) + "M"
     )
     print_line(line)
 
     # swap usage
-    vmem = psutil.virtmem_usage()
-    dashes, empty_dashes = get_dashes(vmem.percent)
+    swap = psutil.swap_memory()
+    dashes, empty_dashes = get_dashes(swap.percent)
     line = " Swap  [%s%s] %5s%% %6s/%s" % (
         dashes, empty_dashes,
-        vmem.percent,
-        str(int(vmem.used / 1024 / 1024)) + "M",
-        str(int(vmem.total / 1024 / 1024)) + "M"
+        swap.percent,
+        str(int(swap.used / 1024 / 1024)) + "M",
+        str(int(swap.total / 1024 / 1024)) + "M"
     )
     print_line(line)
 
@@ -160,19 +156,28 @@ def refresh_window(procs, procs_status):
     for p in procs:
         # TIME+ column shows process CPU cumulative time and it
         # is expressed as: "mm:ss.ms"
-        ctime = timedelta(seconds=sum(p.dict['cpu_times']))
-        ctime = "%s:%s.%s" % (ctime.seconds // 60 % 60,
-                              str((ctime.seconds % 60)).zfill(2),
-                              str(ctime.microseconds)[:2])
+        if p.dict['cpu_times'] != None:
+            ctime = timedelta(seconds=sum(p.dict['cpu_times']))
+            ctime = "%s:%s.%s" % (ctime.seconds // 60 % 60,
+                                  str((ctime.seconds % 60)).zfill(2),
+                                  str(ctime.microseconds)[:2])
+        else:
+            ctime = ''
+        if p.dict['memory_percent'] is not None:
+            p.dict['memory_percent'] = round(p.dict['memory_percent'], 1)
+        else:
+            p.dict['memory_percent'] = ''
+        if p.dict['cpu_percent'] is None:
+            p.dict['cpu_percent'] = ''
         line = templ % (p.pid,
                         p.dict['username'][:8],
                         p.dict['nice'],
-                        bytes2human(p.dict['memory_info'].vms),
-                        bytes2human(p.dict['memory_info'].rss),
+                        bytes2human(getattr(p.dict['memory_info'], 'vms', 0)),
+                        bytes2human(getattr(p.dict['memory_info'], 'rss', 0)),
                         p.dict['cpu_percent'],
-                        round(p.dict['memory_percent'], 1),
+                        p.dict['memory_percent'],
                         ctime,
-                        p.dict['name'],
+                        p.dict['name'] or '',
                         )
         try:
             print_line(line)
