@@ -14,7 +14,7 @@ Python.
 
 from __future__ import division
 
-__version__ = "0.5.0"
+__version__ = "0.6.0"
 version_info = tuple([int(num) for num in __version__.split('.')])
 
 __all__ = [
@@ -30,7 +30,7 @@ __all__ = [
     "Process", "Popen",
     # functions
     "test", "pid_exists", "get_pid_list", "process_iter", "get_process_list",
-    "phymem_usage", "virtmem_usage"
+    "virtual_memory", "swap_memory",
     "cpu_times", "cpu_percent", "per_cpu_percent",
     "network_io_counters", "disk_io_counters",
     ]
@@ -42,6 +42,7 @@ import signal
 import warnings
 import errno
 import subprocess
+import functools
 try:
     import pwd
 except ImportError:
@@ -50,8 +51,11 @@ except ImportError:
 from psutil.error import Error, NoSuchProcess, AccessDenied, TimeoutExpired
 from psutil._compat import property, defaultdict
 from psutil._common import cached_property
-from psutil._common import (nt_disk_iostat as _nt_disk_iostat,
-                            nt_net_iostat as _nt_net_iostat)
+from psutil._common import (deprecated as _deprecated,
+                            nt_disk_iostat as _nt_disk_iostat,
+                            nt_net_iostat as _nt_net_iostat,
+                            nt_sysmeminfo as _nt_sysmeminfo,
+                            isfile_strict as _isfile_strict)
 from psutil._common import (STATUS_RUNNING, STATUS_IDLE, STATUS_SLEEPING,
                             STATUS_DISK_SLEEP, STATUS_STOPPED,
                             STATUS_TRACING_STOP, STATUS_ZOMBIE, STATUS_DEAD,
@@ -91,7 +95,7 @@ __all__.extend(_psplatform.__extra__all__)
 
 NUM_CPUS = _psplatform.NUM_CPUS
 BOOT_TIME = _psplatform.BOOT_TIME
-TOTAL_PHYMEM = _psplatform.phymem_usage()[0]
+TOTAL_PHYMEM = _psplatform.TOTAL_PHYMEM
 
 
 class Process(object):
@@ -212,11 +216,15 @@ class Process(object):
             # If it matches the first part of the cmdline we return that
             # one instead because it's usually more explicative.
             # Examples are "gnome-keyring-d" vs. "gnome-keyring-daemon".
-            cmdline = self.cmdline
-            if cmdline:
-                extended_name = os.path.basename(cmdline[0])
-                if extended_name.startswith(name):
-                    name = extended_name
+            try:
+                cmdline = self.cmdline
+            except AccessDenied:
+                pass
+            else:
+                if cmdline:
+                    extended_name = os.path.basename(cmdline[0])
+                    if extended_name.startswith(name):
+                        name = extended_name
         # XXX - perhaps needs refactoring
         self._platform_impl._process_name = name
         return name
@@ -230,7 +238,8 @@ class Process(object):
             cmdline = self.cmdline
             if cmdline and hasattr(os, 'access') and hasattr(os, 'X_OK'):
                 _exe = os.path.realpath(cmdline[0])
-                if os.path.isfile(_exe) and os.access(_exe, os.X_OK):
+                if os.path.isabs(_exe) and _isfile_strict(_exe) \
+                  and os.access(_exe, os.X_OK):
                     return _exe
         if not exe:
             raise AccessDenied(self.pid, self._platform_impl._process_name)
@@ -289,14 +298,11 @@ class Process(object):
         """
         return self._platform_impl.get_process_create_time()
 
-    # available for Windows and Linux only
-    if hasattr(_psplatform.Process, "get_process_cwd"):
-
-        def getcwd(self):
-            """Return a string representing the process current working
-            directory.
-            """
-            return self._platform_impl.get_process_cwd()
+    def getcwd(self):
+        """Return a string representing the process current working
+        directory.
+        """
+        return self._platform_impl.get_process_cwd()
 
     # Linux, BSD and Windows only
     if hasattr(_psplatform.Process, "get_process_io_counters"):
@@ -361,6 +367,12 @@ class Process(object):
             """
             return self._platform_impl.get_num_fds()
 
+    def get_num_ctx_switches(self):
+        """Return the number voluntary and involuntary context switches
+        performed by this process.
+        """
+        return self._platform_impl.get_num_ctx_switches()
+
     def get_num_threads(self):
         """Return the number of threads used by this process."""
         return self._platform_impl.get_process_num_threads()
@@ -404,7 +416,10 @@ class Process(object):
             for p in process_iter():
                 try:
                     if p.ppid == self.pid:
-                        ret.append(p)
+                        # if child happens to be older than its parent
+                        # (self) it means child's PID has been reused
+                        if self.create_time <= p.create_time:
+                            ret.append(p)
                 except NoSuchProcess:
                     pass
         else:
@@ -422,10 +437,18 @@ class Process(object):
             # to a recursive function call.
             checkpids = [self.pid]
             for pid in checkpids:
-                for proc in table[pid]:
-                    ret.append(proc)
-                    if proc.pid not in checkpids:
-                        checkpids.append(proc.pid)
+                for child in table[pid]:
+                    try:
+                        # if child happens to be older than its parent
+                        # (self) it means child's PID has been reused
+                        intime = self.create_time <= child.create_time
+                    except NoSuchProcess:
+                        pass
+                    else:
+                        if intime:
+                            ret.append(child)
+                            if child.pid not in checkpids:
+                                checkpids.append(child.pid)
         return ret
 
     def get_cpu_percent(self, interval=0.1):
@@ -497,6 +520,13 @@ class Process(object):
         """
         return self._platform_impl.get_memory_info()
 
+    def get_ext_memory_info(self):
+        """Return a namedtuple with variable fields depending on the
+        platform representing extended memory information about
+        the process. All numbers are expressed in bytes.
+        """
+        return self._platform_impl.get_ext_memory_info()
+
     def get_memory_percent(self):
         """Compare physical system memory to process resident memory and
         calculate process memory utilization as a percentage.
@@ -555,6 +585,7 @@ class Process(object):
         udp             UDP
         udp4            UDP over IPv4
         udp6            UDP over IPv6
+        unix            UNIX socket (both UDP and TCP protocols)
         all             the sum of all the possible families and protocols
         """
         return self._platform_impl.get_connections(kind)
@@ -733,6 +764,10 @@ class Popen(Process):
                                       %(self.__class__.__name__, name))
 
 
+# =====================================================================
+# --- system processes related functions
+# =====================================================================
+
 get_pid_list = _psplatform.get_pid_list
 pid_exists = _psplatform.pid_exists
 
@@ -744,6 +779,9 @@ def process_iter():
 
     Every new Process instance is only created once and then cached
     into an internal table which is updated every time this is used.
+
+    The sorting order in which processes are yielded is based on
+    their PIDs.
     """
     def add(pid):
         proc = Process(pid)
@@ -760,14 +798,18 @@ def process_iter():
 
     for pid in gone_pids:
         remove(pid)
-    for pid, proc in list(_pmap.items()):
+    for pid, proc in sorted(list(_pmap.items()) + \
+                            list(dict.fromkeys(new_pids).items())):
         try:
-            # use is_running() to check whether PID has been reused by
-            # another process in which case yield a new Process instance
-            if proc.is_running():
-                yield proc
-            else:
+            if proc is None:  # new process
                 yield add(pid)
+            else:
+                # use is_running() to check whether PID has been reused by
+                # another process in which case yield a new Process instance
+                if proc.is_running():
+                    yield proc
+                else:
+                    yield add(pid)
         except NoSuchProcess:
             remove(pid)
         except AccessDenied:
@@ -775,11 +817,10 @@ def process_iter():
             # no way to tell whether the pid of the cached process
             # has been reused. Just return the cached version.
             yield proc
-    for pid in new_pids:
-        try:
-            yield add(pid)
-        except NoSuchProcess:
-            pass
+
+# =====================================================================
+# --- CPU related functions
+# =====================================================================
 
 def cpu_times(percpu=False):
     """Return system-wide CPU times as a namedtuple object.
@@ -868,22 +909,83 @@ def cpu_percent(interval=0.1, percpu=False):
             ret.append(calculate(t1, t2))
         return ret
 
-def phymem_usage():
-    """Return the amount of total, used and free physical memory
-    on the system in bytes plus the percentage usage.
-    """
-    return _psplatform.phymem_usage()
+# =====================================================================
+# --- system memory related functions
+# =====================================================================
 
-def virtmem_usage():
-    """Return the amount of total, used and free virtual memory
-    on the system in bytes plus the percentage usage.
+def virtual_memory():
+    """Return statistics about system memory usage as a namedtuple
+    including the following fields, expressed in bytes:
 
-    On Linux they match the values returned by free command line utility.
-    On OS X and FreeBSD they represent the same values as returned by
-    sysctl vm.vmtotal. On Windows they are determined by reading the
-    PageFile values of MEMORYSTATUSEX structure.
+     - total:
+       total physical memory available.
+
+     - available:
+       the actual amount of available memory that can be given
+       instantly to processes that request more memory in bytes; this
+       is calculated by summing different memory values depending on
+       the platform (e.g. free + buffers + cached on Linux) and it is
+       supposed to be used to monitor actual memory usage in a cross
+       platform fashion.
+
+     - percent:
+       the percentage usage calculated as (total - available) / total * 100
+
+     - used:
+       memory used, calculated differently depending on the platform and
+       designed for informational purposes only:
+        OSX: active + inactive + wired
+        BSD: active + wired + cached
+        LINUX: total - free
+
+     - free:
+       memory not being used at all (zeroed) that is readily available;
+       note that this doesn't reflect the actual memory available
+       (use 'available' instead)
+
+    Platform-specific fields:
+
+     - active (UNIX):
+       memory currently in use or very recently used, and so it is in RAM.
+
+     - inactive (UNIX):
+       memory that is marked as not used.
+
+     - buffers (BSD, Linux):
+       cache for things like file system metadata.
+
+     - cached (BSD, OSX):
+       cache for various things.
+
+     - wired (OSX, BSD):
+       memory that is marked to always stay in RAM. It is never moved to disk.
+
+     - shared (BSD):
+       memory that may be simultaneously accessed by multiple processes.
+
+    The sum of 'used' and 'available' does not necessarily equal total.
+    On Windows 'available' and 'free' are the same.
     """
-    return _psplatform.virtmem_usage()
+    return _psplatform.virtual_memory()
+
+def swap_memory():
+    """Return system swap memory statistics as a namedtuple including
+    the following attributes:
+
+     - total:   total swap memory in bytes
+     - used:    used swap memory in bytes
+     - free:    free swap memory in bytes
+     - percent: the percentage usage
+     - sin:     no. of bytes the system has swapped in from disk (cumulative)
+     - sout:    no. of bytes the system has swapped out from disk (cumulative)
+
+    'sin' and 'sout' on Windows are meaningless and always set to 0.
+    """
+    return _psplatform.swap_memory()
+
+# =====================================================================
+# --- disks/paritions related functions
+# =====================================================================
 
 def disk_usage(path):
     """Return disk usage statistics about the given path as a namedtuple
@@ -902,30 +1004,6 @@ def disk_partitions(all=False):
     """
     return _psplatform.disk_partitions(all)
 
-def network_io_counters(pernic=False):
-    """Return network I/O statistics as a namedtuple including
-    the following attributes:
-
-     - bytes_sent:   number of bytes sent
-     - bytes_recv:   number of bytes received
-     - packets_sent: number of packets sent
-     - packets_recv: number of packets received
-
-    If pernic is True return the same information for every
-    network interface installed on the system as a dictionary
-    with network interface names as the keys and the namedtuple
-    described above as the values.
-    """
-    rawdict = _psplatform.network_io_counters()
-    if not rawdict:
-        raise RuntimeError("couldn't find any network interface")
-    if pernic:
-        for nic, fields in rawdict.items():
-            rawdict[nic] = _nt_net_iostat(*fields)
-        return rawdict
-    else:
-        return _nt_net_iostat(*[sum(x) for x in zip(*rawdict.values())])
-
 def disk_io_counters(perdisk=False):
     """Return system disk I/O statistics as a namedtuple including
     the following attributes:
@@ -952,6 +1030,43 @@ def disk_io_counters(perdisk=False):
     else:
         return _nt_disk_iostat(*[sum(x) for x in zip(*rawdict.values())])
 
+# =====================================================================
+# --- network related functions
+# =====================================================================
+
+def network_io_counters(pernic=False):
+    """Return network I/O statistics as a namedtuple including
+    the following attributes:
+
+     - bytes_sent:   number of bytes sent
+     - bytes_recv:   number of bytes received
+     - packets_sent: number of packets sent
+     - packets_recv: number of packets received
+     - errin:        total number of errors while receiving
+     - errout:       total number of errors while sending
+     - dropin:       total number of incoming packets which were dropped
+     - dropout:      total number of outgoing packets which were dropped
+                     (always 0 on OSX and BSD)
+
+    If pernic is True return the same information for every
+    network interface installed on the system as a dictionary
+    with network interface names as the keys and the namedtuple
+    described above as the values.
+    """
+    rawdict = _psplatform.network_io_counters()
+    if not rawdict:
+        raise RuntimeError("couldn't find any network interface")
+    if pernic:
+        for nic, fields in rawdict.items():
+            rawdict[nic] = _nt_net_iostat(*fields)
+        return rawdict
+    else:
+        return _nt_net_iostat(*[sum(x) for x in zip(*rawdict.values())])
+
+# =====================================================================
+# --- other system related functions
+# =====================================================================
+
 def get_users():
     """Return users currently connected on the system as a list of
     namedtuples including the following attributes.
@@ -964,20 +1079,9 @@ def get_users():
     """
     return _psplatform.get_system_users()
 
-# http://goo.gl/jYLvf
-def _deprecated(replacement=None):
-    """A decorator which can be used to mark functions as deprecated."""
-    def outer(fun):
-        def inner(*args, **kwargs):
-            msg = "psutil.%s is deprecated" % fun.__name__
-            if replacement is not None:
-                msg += "; use %s instead" % replacement
-            warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
-            return fun(*args, **kwargs)
-        return inner
-    return outer
-
+# =====================================================================
 # --- deprecated functions
+# =====================================================================
 
 @_deprecated()
 def get_process_list():
@@ -986,23 +1090,36 @@ def get_process_list():
     """
     return list(process_iter())
 
-@_deprecated("psutil.phymem_usage")
+@_deprecated()
+def phymem_usage():
+    """Return the amount of total, used and free physical memory
+    on the system in bytes plus the percentage usage.
+    Deprecated by psutil.virtual_memory().
+    """
+    mem = virtual_memory()
+    return _nt_sysmeminfo(mem.total, mem.used, mem.free, mem.percent)
+
+@_deprecated("psutil.swap_memory()")
+def virtmem_usage():
+    return swap_memory()
+
+@_deprecated("psutil.phymem_usage().free")
 def avail_phymem():
     return phymem_usage().free
 
-@_deprecated("psutil.phymem_usage")
+@_deprecated("psutil.phymem_usage().used")
 def used_phymem():
     return phymem_usage().used
 
-@_deprecated("psutil.virtmem_usage")
+@_deprecated("psutil.virtmem_usage().total")
 def total_virtmem():
     return virtmem_usage().total
 
-@_deprecated("psutil.virtmem_usage")
+@_deprecated("psutil.virtmem_usage().used")
 def used_virtmem():
     return virtmem_usage().used
 
-@_deprecated("psutil.virtmem_usage")
+@_deprecated("psutil.virtmem_usage().free")
 def avail_virtmem():
     return virtmem_usage().free
 
