@@ -9,6 +9,13 @@
 """Common objects shared by all _ps* modules."""
 
 from __future__ import division
+import sys
+import os
+import stat
+import errno
+import functools
+import warnings
+
 from psutil._compat import namedtuple, long
 
 # --- functions
@@ -24,7 +31,6 @@ def usage_percent(used, total, _round=None):
     else:
         return ret
 
-
 class constant(int):
     """A constant type; overrides base int to provide a useful name on str()."""
 
@@ -68,12 +74,50 @@ def memoize(f):
 
 class cached_property(object):
     """A memoize decorator for class properties."""
+    enabled = True
+
     def __init__(self, func):
         self.func = func
 
     def __get__(self, instance, type):
-        res = instance.__dict__[self.func.__name__] = self.func(instance)
-        return res
+        ret = self.func(instance)
+        if self.enabled:
+            instance.__dict__[self.func.__name__] = ret
+        return ret
+
+# http://goo.gl/jYLvf
+def deprecated(replacement=None):
+    """A decorator which can be used to mark functions as deprecated."""
+    def outer(fun):
+        msg = "psutil.%s is deprecated" % fun.__name__
+        if replacement is not None:
+            msg += "; use %s instead" % replacement
+        if fun.__doc__ is None:
+            fun.__doc__ = msg
+
+        @functools.wraps(fun)
+        def inner(*args, **kwargs):
+            warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
+            return fun(*args, **kwargs)
+
+        return inner
+    return outer
+
+
+def isfile_strict(path):
+    """Same as os.path.isfile() but does not swallow EACCES / EPERM
+    exceptions, see:
+    http://mail.python.org/pipermail/python-dev/2012-June/120787.html
+    """
+    try:
+        st = os.stat(path)
+    except OSError:
+        err = sys.exc_info()[1]
+        if err.errno in (errno.EPERM, errno.EACCES):
+            raise
+        return False
+    else:
+        return stat.S_ISREG(st.st_mode)
 
 
 # --- constants
@@ -96,9 +140,10 @@ STATUS_WAITING = constant(11, "waiting")  # BSD
 import socket
 from socket import AF_INET, SOCK_STREAM, SOCK_DGRAM
 AF_INET6 = getattr(socket, 'AF_INET6', None)
+AF_UNIX = getattr(socket, 'AF_UNIX', None)
 
 conn_tmap = {
-    "all"  : ([AF_INET, AF_INET6], [SOCK_STREAM, SOCK_DGRAM]),
+    "all"  : ([AF_INET, AF_INET6, AF_UNIX], [SOCK_STREAM, SOCK_DGRAM]),
     "tcp"  : ([AF_INET, AF_INET6], [SOCK_STREAM]),
     "tcp4" : ([AF_INET],           [SOCK_STREAM]),
     "udp"  : ([AF_INET, AF_INET6], [SOCK_DGRAM]),
@@ -114,16 +159,25 @@ if AF_INET6 is not None:
         "udp6" : ([AF_INET6],          [SOCK_DGRAM]),
     })
 
-del AF_INET, AF_INET6, SOCK_STREAM, SOCK_DGRAM, socket
+if AF_UNIX is not None:
+    conn_tmap.update({
+        "unix" : ([AF_UNIX],           [SOCK_STREAM, SOCK_DGRAM]),
+    })
+
+
+del AF_INET, AF_INET6, AF_UNIX, SOCK_STREAM, SOCK_DGRAM, socket
 
 # --- namedtuples
 
 # system
 nt_sys_cputimes = namedtuple('cputimes', 'user nice system idle iowait irq softirq')
 nt_sysmeminfo = namedtuple('usage', 'total used free percent')
+# XXX - would 'available' be better than 'free' as for virtual_memory() nt?
+nt_swapmeminfo = namedtuple('swap', 'total used free percent sin sout')
 nt_diskinfo = namedtuple('usage', 'total used free percent')
 nt_partition = namedtuple('partition',  'device mountpoint fstype opts')
-nt_net_iostat = namedtuple('iostat', 'bytes_sent bytes_recv packets_sent packets_recv')
+nt_net_iostat = namedtuple('iostat',
+    'bytes_sent bytes_recv packets_sent packets_recv errin errout dropin dropout')
 nt_disk_iostat = namedtuple('iostat', 'read_count write_count read_bytes write_bytes read_time write_time')
 nt_user = namedtuple('user', 'name terminal host started')
 
@@ -137,3 +191,4 @@ nt_uids = namedtuple('user', 'real effective saved')
 nt_gids = namedtuple('group', 'real effective saved')
 nt_io = namedtuple('io', 'read_count write_count read_bytes write_bytes')
 nt_ionice = namedtuple('ionice', 'ioclass value')
+nt_ctxsw = namedtuple('amount', 'voluntary involuntary')
