@@ -8,16 +8,68 @@
 
 """Module which provides compatibility with older Python versions."""
 
-__all__ = ["namedtuple", "property"]
+__all__ = ["PY3", "int", "long", "xrange", "exec_", "callable",
+           "namedtuple", "property", "defaultdict"]
 
-from operator import itemgetter as _itemgetter
-from keyword import iskeyword as _iskeyword
-import sys as _sys
-import __builtin__
+import sys
+
+
+# --- python 2/3 compatibility layer
+
+PY3 = sys.version_info >= (3,)
+
+try:
+    import __builtin__
+except ImportError:
+    import builtins as __builtin__  # py3
+
+if PY3:
+    int = int
+    long = int
+    xrange = range
+    exec_ = getattr(__builtin__, "exec")
+    print_ = getattr(__builtin__, "print")
+else:
+    int = int
+    long = long
+    xrange = xrange
+
+    def exec_(code, globs=None, locs=None):
+        if globs is None:
+            frame = _sys._getframe(1)
+            globs = frame.f_globals
+            if locs is None:
+                locs = frame.f_locals
+            del frame
+        elif locs is None:
+            locs = globs
+        exec("""exec code in globs, locs""")
+
+    def print_(s):
+        sys.stdout.write(s + '\n')
+        sys.stdout.flush()
+
+
+# removed in 3.0, reintroduced in 3.2
+try:
+    callable = callable
+except Exception:
+    def callable(obj):
+        for klass in type(obj).__mro__:
+            if "__call__" in klass.__dict__:
+                return True
+        return False
+
+
+# --- stdlib additions
 
 try:
     from collections import namedtuple
 except ImportError:
+    from operator import itemgetter as _itemgetter
+    from keyword import iskeyword as _iskeyword
+    import sys as _sys
+
     def namedtuple(typename, field_names, verbose=False, rename=False):
         """A collections.namedtuple implementation written in Python
         to support Python versions < 2.6.
@@ -95,14 +147,16 @@ except ImportError:
         for i, name in enumerate(field_names):
             template += '        %s = _property(_itemgetter(%d))\n' % (name, i)
         if verbose:
-            print template
+            sys.stdout.write(template + '\n')
+            sys.stdout.flush()
 
         # Execute the template string in a temporary namespace
         namespace = dict(_itemgetter=_itemgetter, __name__='namedtuple_%s' % typename,
                          _property=property, _tuple=tuple)
         try:
-            exec template in namespace
-        except SyntaxError, e:
+            exec_(template, namespace)
+        except SyntaxError:
+            e = sys.exc_info()[1]
             raise SyntaxError(e.message + ':\n' + template)
         result = namespace[typename]
 
@@ -139,3 +193,54 @@ else:
 
         def deleter(self, method):
             return property(self.fget, self.fset, method)
+
+
+# py 2.5 collections.defauldict
+# Taken from:
+# http://code.activestate.com/recipes/523034-emulate-collectionsdefaultdict/
+# credits: Jason Kirtland
+try:
+    from collections import defaultdict
+except ImportError:
+    class defaultdict(dict):
+
+        def __init__(self, default_factory=None, *a, **kw):
+            if (default_factory is not None and
+                not hasattr(default_factory, '__call__')):
+                raise TypeError('first argument must be callable')
+            dict.__init__(self, *a, **kw)
+            self.default_factory = default_factory
+
+        def __getitem__(self, key):
+            try:
+                return dict.__getitem__(self, key)
+            except KeyError:
+                return self.__missing__(key)
+
+        def __missing__(self, key):
+            if self.default_factory is None:
+                raise KeyError(key)
+            self[key] = value = self.default_factory()
+            return value
+
+        def __reduce__(self):
+            if self.default_factory is None:
+                args = tuple()
+            else:
+                args = self.default_factory,
+            return type(self), args, None, None, self.items()
+
+        def copy(self):
+            return self.__copy__()
+
+        def __copy__(self):
+            return type(self)(self.default_factory, self)
+
+        def __deepcopy__(self, memo):
+            import copy
+            return type(self)(self.default_factory,
+                              copy.deepcopy(self.items()))
+
+        def __repr__(self):
+            return 'defaultdict(%s, %s)' % (self.default_factory,
+                                            dict.__repr__(self))
