@@ -23,7 +23,7 @@ import _psutil_linux
 from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._common import *
-from psutil._compat import PY3, xrange, long
+from psutil._compat import PY3, xrange, long, namedtuple
 
 __extra__all__ = [
     "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE",
@@ -87,6 +87,7 @@ _PAGESIZE = os.sysconf("SC_PAGE_SIZE")
 _TERMINAL_MAP = _psposix._get_terminal_map()
 BOOT_TIME = _get_boot_time()
 NUM_CPUS = _get_num_cpus()
+TOTAL_PHYMEM = _psutil_linux.get_sysinfo()[0]
 # ioprio_* constants http://linux.die.net/man/2/ioprio_get
 IOPRIO_CLASS_NONE = 0
 IOPRIO_CLASS_RT = 1
@@ -109,57 +110,72 @@ _TCP_STATES_TABLE = {"01" : "ESTABLISHED",
 
 # --- system memory functions
 
-def cached_phymem():
-    """Return the amount of cached memory on the system, in bytes.
-    This reflects the "cached" column of free command line utility.
-    """
+nt_virtmem_info = namedtuple('vmem', ' '.join([
+    # all platforms
+    'total', 'available', 'percent', 'used', 'free',
+    # linux specific
+    'active',
+    'inactive',
+    'buffers',
+    'cached']))
+
+def virtual_memory():
+    total, free, buffers, shared, _, _ = _psutil_linux.get_sysinfo()
+    cached = active = inactive = None
     f = open('/proc/meminfo', 'r')
     try:
         for line in f:
             if line.startswith('Cached:'):
-                return int(line.split()[1]) * 1024
-        raise RuntimeError("line not found")
-    finally:
-        f.close()
-
-def phymem_buffers():
-    """Return the amount of physical memory buffers used by the
-    kernel in bytes.
-    This reflects the "buffers" column of free command line utility.
-    """
-    total, free, buffers = _psutil_linux.get_physmem()
-    return buffers
-
-def phymem_usage():
-    """Return physical memory usage statistics as a namedutple including
-    tota, used, free and percent usage.
-    """
-    # total, used and free values are matched against free cmdline utility
-    # the percentage matches top/htop and gnome-system-monitor
-    total, free, buffers = _psutil_linux.get_physmem()
-    cached = cached_phymem()
-    used = total - free
-    percent = usage_percent(total - (free + buffers + cached), total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
-
-
-def virtmem_usage():
-    f = open('/proc/meminfo', 'r')
-    try:
-        total = free = None
-        for line in f:
-            if line.startswith('SwapTotal:'):
-                total = int(line.split()[1]) * 1024
-            elif line.startswith('SwapFree:'):
-                free = int(line.split()[1]) * 1024
-            if total is not None and free is not None:
+                cached = int(line.split()[1]) * 1024
+            elif line.startswith('Active:'):
+                active = int(line.split()[1]) * 1024
+            elif line.startswith('Inactive:'):
+                inactive = int(line.split()[1]) * 1024
+            if cached is not None \
+            and active is not None \
+            and inactive is not None:
                 break
-        assert total is not None and free is not None
-        used = total - free
-        percent = usage_percent(used, total, _round=1)
-        return nt_sysmeminfo(total, used, free, percent)
+        else:
+            raise RuntimeError("line(s) not found")
     finally:
         f.close()
+    avail = free + buffers + cached
+    used = total - free
+    percent = usage_percent((total - avail), total, _round=1)
+    return nt_virtmem_info(total, avail, percent, used, free,
+                           active, inactive, buffers, cached)
+
+def swap_memory():
+    _, _, _, _, total, free = _psutil_linux.get_sysinfo()
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    # get pgin/pgouts
+    f = open("/proc/vmstat", "r")
+    sin = sout = None
+    try:
+        for line in f:
+            # values are expressed in 4 kilo bytes, we want bytes instead
+            if line.startswith('pswpin'):
+                sin = int(line.split(' ')[1]) * 4 * 1024
+            elif line.startswith('pswpout'):
+                sout = int(line.split(' ')[1])  * 4 * 1024
+            if sin is not None and sout is not None:
+                break
+        else:
+            raise RuntimeError("line(s) not found")
+    finally:
+        f.close()
+    return nt_swapmeminfo(total, used, free, percent, sin, sout)
+
+# --- XXX deprecated memory functions
+
+@deprecated('psutil.virtual_memory().cached')
+def cached_phymem():
+    return virtual_memory().cached
+
+@deprecated('psutil.virtual_memory().buffers')
+def phymem_buffers():
+    return virtual_memory().buffers
 
 
 # --- system CPU functions
@@ -275,9 +291,14 @@ def network_io_counters():
         fields = line[colon+1:].strip().split()
         bytes_recv = int(fields[0])
         packets_recv = int(fields[1])
+        errin = int(fields[2])
+        dropin = int(fields[2])
         bytes_sent = int(fields[8])
         packets_sent = int(fields[9])
-        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv)
+        errout = int(fields[10])
+        dropout = int(fields[11])
+        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv,
+                         errin, errout, dropin, dropout)
     return retdict
 
 def disk_io_counters():
@@ -407,7 +428,7 @@ class Process(object):
         # However, if we had permissions to execute readlink() it's
         # likely that we'll be able to figure out exe from argv[0]
         # later on.
-        if exe.endswith(" (deleted)") and not os.path.isfile(exe):
+        if exe.endswith(" (deleted)") and not isfile_strict(exe):
             return ""
         return exe
 
@@ -501,6 +522,29 @@ class Process(object):
         finally:
             f.close()
 
+    _nt_ext_mem = namedtuple('meminfo', 'rss vms shared text lib data dirty')
+
+    @wrap_exceptions
+    def get_ext_memory_info(self):
+        #  ============================================================
+        # | FIELD  | DESCRIPTION                         | AKA  | TOP  |
+        #  ============================================================
+        # | rss    | resident set size                   |      | RES  |
+        # | vms    | total program size                  | size | VIRT |
+        # | shared | shared pages (from shared mappings) |      | SHR  |
+        # | text   | text ('code')                       | trs  | CODE |
+        # | lib    | library (unused in Linux 2.6)       | lrs  |      |
+        # | data   | data + stack                        | drs  | DATA |
+        # | dirty  | dirty pages (unused in Linux 2.6)   | dt   |      |
+        #  ============================================================
+        f = open("/proc/%s/statm" % self.pid)
+        try:
+            vms, rss, shared, text, lib, data, dirty = \
+              [int(x) * _PAGESIZE for x in f.readline().split()[:7]]
+        finally:
+            f.close()
+        return self._nt_ext_mem(rss, vms, shared, text, lib, data, dirty)
+
     _mmap_base_fields = ['path', 'rss', 'size', 'pss', 'shared_clean',
                          'shared_dirty', 'private_clean', 'private_dirty',
                          'referenced', 'anonymous', 'swap',]
@@ -582,6 +626,22 @@ class Process(object):
         path = os.readlink("/proc/%s/cwd" % self.pid)
         return path.replace('\x00', '')
 
+    @wrap_exceptions
+    def get_num_ctx_switches(self):
+        vol = unvol = None
+        f = open("/proc/%s/status" % self.pid)
+        try:
+            for line in f:
+                if line.startswith("voluntary_ctxt_switches"):
+                    vol = int(line.split()[1])
+                elif line.startswith("nonvoluntary_ctxt_switches"):
+                    unvol = int(line.split()[1])
+                if vol is not None and unvol is not None:
+                    return nt_ctxsw(vol, unvol)
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
+
     @wrap_exceptions
     def get_process_num_threads(self):
         f = open("/proc/%s/status" % self.pid)
@@ -735,7 +795,7 @@ class Process(object):
                     # to tell whether it's a regular file or not,
                     # so we skip it. A regular file is always supposed
                     # to be absolutized though.
-                    if file.startswith('/') and os.path.isfile(file):
+                    if file.startswith('/') and isfile_strict(file):
                         ntuple = nt_openfile(file, int(fd))
                         retlist.append(ntuple)
         if hit_enoent:
@@ -761,6 +821,9 @@ class Process(object):
         udp6            UDP over IPv6
         all             the sum of all the possible families and protocols
         """
+        # Note: in case of UNIX sockets we're only able to determine the
+        # local bound path while the remote endpoint is not retrievable:
+        # http://goo.gl/R3GHM
         inodes = {}
         # os.listdir() is gonna raise a lot of access denied
         # exceptions in case of unprivileged user; that's fine:
@@ -808,6 +871,20 @@ class Process(object):
                             conn = nt_connection(fd, family, type_, laddr,
                                                  raddr, status)
                             retlist.append(conn)
+                    elif family == socket.AF_UNIX:
+                        tokens = line.split()
+                        _, _, _, _, type_, _, inode = tokens[0:7]
+                        if inode in inodes:
+
+                            if len(tokens) == 8:
+                                path = tokens[-1]
+                            else:
+                                path = ""
+                            fd = int(inodes[inode])
+                            type_ = int(type_)
+                            conn = nt_connection(fd, family, type_, path,
+                                                 None, "")
+                            retlist.append(conn)
                     else:
                         raise ValueError(family)
                 return retlist
@@ -818,15 +895,17 @@ class Process(object):
         tcp6 = ("tcp6", socket.AF_INET6, socket.SOCK_STREAM)
         udp4 = ("udp" , socket.AF_INET , socket.SOCK_DGRAM)
         udp6 = ("udp6", socket.AF_INET6, socket.SOCK_DGRAM)
+        unix = ("unix", socket.AF_UNIX, None)
 
         tmap = {
-            "all"  : (tcp4, tcp6, udp4, udp6),
+            "all"  : (tcp4, tcp6, udp4, udp6, unix),
             "tcp"  : (tcp4, tcp6),
             "tcp4" : (tcp4,),
             "tcp6" : (tcp6,),
             "udp"  : (udp4, udp6),
             "udp4" : (udp4,),
             "udp6" : (udp6,),
+            "unix" : (unix,),
             "inet" : (tcp4, tcp6, udp4, udp6),
             "inet4": (tcp4, udp4),
             "inet6": (tcp6, udp6),
