@@ -25,26 +25,35 @@ __extra__all__ = []
 
 NUM_CPUS = _psutil_osx.get_num_cpus()
 BOOT_TIME = _psutil_osx.get_system_boot_time()
+TOTAL_PHYMEM = _psutil_osx.get_virtual_mem()[0]
+_PAGESIZE = os.sysconf("SC_PAGE_SIZE")
 _TERMINAL_MAP = _psposix._get_terminal_map()
 _cputimes_ntuple = namedtuple('cputimes', 'user nice system idle')
 
 # --- functions
 
-def phymem_usage():
-    """Physical system memory as a (total, used, free) tuple."""
-    total = _psutil_osx.get_total_phymem()
-    free =  _psutil_osx.get_avail_phymem()
-    used = total - free
-    percent = usage_percent(used, total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
+nt_virtmem_info = namedtuple('vmem', ' '.join([
+    # all platforms
+    'total', 'available', 'percent', 'used', 'free',
+    # OSX specific
+    'active',
+    'inactive',
+    'wired']))
 
-def virtmem_usage():
-    """Virtual system memory as a (total, used, free) tuple."""
-    total = _psutil_osx.get_total_virtmem()
-    free =  _psutil_osx.get_avail_virtmem()
-    used = total - free
+def virtual_memory():
+    """System virtual memory as a namedtuple."""
+    total, active, inactive, wired, free = _psutil_osx.get_virtual_mem()
+    avail = inactive + free
+    used = active + inactive + wired
+    percent = usage_percent((total - avail), total, _round=1)
+    return nt_virtmem_info(total, avail, percent, used, free,
+                           active, inactive, wired)
+
+def swap_memory():
+    """Swap system memory as a (total, used, free, sin, sout) tuple."""
+    total, used, free, sin, sout = _psutil_osx.get_swap_mem()
     percent = usage_percent(used, total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
+    return nt_swapmeminfo(total, used, free, percent, sin, sout)
 
 def get_system_cpu_times():
     """Return system CPU times as a namedtuple."""
@@ -137,12 +146,9 @@ class Process(object):
         """Return process name as a string of limited len (15)."""
         return _psutil_osx.get_process_name(self.pid)
 
+    @wrap_exceptions
     def get_process_exe(self):
-        # no such thing as "exe" on OS X; it will maybe be determined
-        # later from cmdline[0]
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._process_name)
-        return ""
+        return _psutil_osx.get_process_exe(self.pid)
 
     @wrap_exceptions
     def get_process_cmdline(self):
@@ -156,6 +162,10 @@ class Process(object):
         """Return process parent pid."""
         return _psutil_osx.get_process_ppid(self.pid)
 
+    @wrap_exceptions
+    def get_process_cwd(self):
+        return _psutil_osx.get_process_cwd(self.pid)
+
     @wrap_exceptions
     def get_process_uids(self):
         real, effective, saved = _psutil_osx.get_process_uids(self.pid)
@@ -177,12 +187,22 @@ class Process(object):
     @wrap_exceptions
     def get_memory_info(self):
         """Return a tuple with the process' RSS and VMS size."""
-        rss, vms = _psutil_osx.get_memory_info(self.pid)
+        rss, vms = _psutil_osx.get_process_memory_info(self.pid)[:2]
         return nt_meminfo(rss, vms)
 
+    _nt_ext_mem = namedtuple('meminfo', 'rss vms pfaults pageins')
+
+    @wrap_exceptions
+    def get_ext_memory_info(self):
+        """Return a tuple with the process' RSS and VMS size."""
+        rss, vms, pfaults, pageins = _psutil_osx.get_process_memory_info(self.pid)
+        return self._nt_ext_mem(rss, vms,
+                                pfaults * _PAGESIZE,
+                                pageins * _PAGESIZE)
+
     @wrap_exceptions
     def get_cpu_times(self):
-        user, system = _psutil_osx.get_cpu_times(self.pid)
+        user, system = _psutil_osx.get_process_cpu_times(self.pid)
         return nt_cputimes(user, system)
 
     @wrap_exceptions
@@ -191,6 +211,10 @@ class Process(object):
         the epoch."""
         return _psutil_osx.get_process_create_time(self.pid)
 
+    @wrap_exceptions
+    def get_num_ctx_switches(self):
+        return nt_ctxsw(*_psutil_osx.get_process_num_ctx_switches(self.pid))
+
     @wrap_exceptions
     def get_process_num_threads(self):
         """Return the number of threads belonging to the process."""
@@ -204,7 +228,7 @@ class Process(object):
         files = []
         rawlist = _psutil_osx.get_process_open_files(self.pid)
         for path, fd in rawlist:
-            if os.path.isfile(path):
+            if isfile_strict(path):
                 ntuple = nt_openfile(path, fd)
                 files.append(ntuple)
         return files
