@@ -13,7 +13,7 @@
 static PyObject* linux_ioprio_get(PyObject* self, PyObject* args);
 static PyObject* linux_ioprio_set(PyObject* self, PyObject* args);
 static PyObject* get_disk_partitions(PyObject* self, PyObject* args);
-static PyObject* get_physmem(PyObject* self, PyObject* args);
+static PyObject* get_sysinfo(PyObject* self, PyObject* args);
 static PyObject* get_process_cpu_affinity(PyObject* self, PyObject* args);
 static PyObject* set_process_cpu_affinity(PyObject* self, PyObject* args);
 static PyObject* get_system_users(PyObject* self, PyObject* args);
