@@ -48,32 +48,68 @@ BSD = sys.platform.startswith("freebsd")
 
 _subprocesses_started = set()
 
-def get_test_subprocess(cmd=None, stdout=DEVNULL, stderr=DEVNULL, stdin=None):
+def get_test_subprocess(cmd=None, stdout=DEVNULL, stderr=DEVNULL, stdin=None,
+                        wait=False):
     """Return a subprocess.Popen object to use in tests.
     By default stdout and stderr are redirected to /dev/null and the
     python interpreter is used as test process.
+    If 'wait' is True attemps to make sure the process is in a
+    reasonably initialized state.
     """
     if cmd is None:
-        cmd = [PYTHON, "-c", "import time; time.sleep(3600);"]
-    sproc = subprocess.Popen(cmd, stdout=stdout, stderr=stderr, stdin=stdin)
+        pyline = ""
+        if wait:
+            pyline += "open('%s', 'w'); " % TESTFN
+        pyline += "import time; time.sleep(3600);"
+        cmd_ = [PYTHON, "-c", pyline]
+    else:
+        cmd_ = cmd
+    sproc = subprocess.Popen(cmd_, stdout=stdout, stderr=stderr, stdin=stdin)
+    if wait:
+        if cmd is None:
+            while 1:
+                if os.path.exists(TESTFN):
+                    break
+                time.sleep(0.001)
+        else:
+            wait_for_pid(sproc.pid)
     _subprocesses_started.add(sproc.pid)
     return sproc
 
-def sh(cmdline):
+def warn(msg, warntype=RuntimeWarning):
+    """Raise a warning msg."""
+    warnings.warn(msg, warntype)
+
+def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
     """run cmd in a subprocess and return its output.
     raises RuntimeError on error.
     """
-    p = subprocess.Popen(cmdline, shell=True, stdout=subprocess.PIPE,
-                                              stderr=subprocess.PIPE)
+    p = subprocess.Popen(cmdline, shell=True, stdout=stdout, stderr=stderr)
     stdout, stderr = p.communicate()
     if p.returncode != 0:
         raise RuntimeError(stderr)
     if stderr:
-        warnings.warn(stderr, RuntimeWarning)
+        warn(stderr)
     if PY3:
         stdout = str(stdout, sys.stdout.encoding)
     return stdout.strip()
 
+def which(program):
+    """Same as UNIX which command. Return None on command not found."""
+    def is_exe(fpath):
+        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
+
+    fpath, fname = os.path.split(program)
+    if fpath:
+        if is_exe(program):
+            return program
+    else:
+        for path in os.environ["PATH"].split(os.pathsep):
+            exe_file = os.path.join(path, program)
+            if is_exe(exe_file):
+                return exe_file
+    return None
+
 def wait_for_pid(pid, timeout=1):
     """Wait for pid to show up in the process list then return.
     Used in the test suite to give time the sub process to initialize.
@@ -93,20 +129,22 @@ def reap_children(search_all=False):
     no zombies stick around to hog resources and create problems when
     looking for refleaks.
     """
+    pids = _subprocesses_started
     if search_all:
         this_process = psutil.Process(os.getpid())
-        pids = [x.pid for x in this_process.get_children(recursive=True)]
-    else:
-        pids =_subprocesses_started
+        for p in this_process.get_children(recursive=True):
+            pids.add(p.pid)
     while pids:
         pid = pids.pop()
         try:
             child = psutil.Process(pid)
             child.kill()
-        except (psutil.NoSuchProcess, psutil.AccessDenied):
+        except psutil.NoSuchProcess:
             pass
+        except psutil.AccessDenied:
+            warn("couldn't kill child process with pid %s" % pid)
         else:
-            child.wait()
+            child.wait(timeout=3)
 
 def check_ip_address(addr, family):
     """Attempts to check IP address's validity."""
@@ -121,9 +159,24 @@ def check_ip_address(addr, family):
             assert 0 <= num <= 255, ip
     assert 0 <= port <= 65535, port
 
+def safe_remove(fname):
+    try:
+        os.remove(fname)
+    except OSError:
+        pass
+
+def call_until(fun, expr, timeout=1):
+    """Keep calling function for timeout secs and exit if eval()
+    expression is True.
+    """
+    stop_at = time.time() + timeout
+    while time.time() < stop_at:
+        ret = fun()
+        if eval(expr):
+            return ret
+        time.sleep(0.001)
+    raise RuntimeError('timed out (ret=%r)' % ret)
 
-# we want to search through all processes before exiting
-atexit.register(reap_children, search_all=True)
 
 def skipIf(condition, reason="", warn=False):
     """Decorator which skip a test under if condition is satisfied.
@@ -144,7 +197,7 @@ def skipIf(condition, reason="", warn=False):
                     msg = "%s was skipped" % objname
                     if reason:
                         msg += "; reason: " + repr(reason)
-                    atexit.register(warnings.warn, msg, RuntimeWarning)
+                    atexit.register(warn, msg)
                 return
             else:
                 return fun(self, *args, **kwargs)
@@ -225,6 +278,9 @@ class ThreadTask(threading.Thread):
 
 class TestCase(unittest.TestCase):
 
+    def setUp(self):
+        safe_remove(TESTFN)
+
     def tearDown(self):
         reap_children()
 
@@ -234,6 +290,14 @@ class TestCase(unittest.TestCase):
                 self.fail(msg or \
                         '%s not found in %s' % (repr(member), repr(container)))
 
+    def assert_eq_w_tol(self, first, second, tolerance):
+        difference = abs(first - second)
+        if difference <= tolerance:
+            return
+        msg = '%r != %r within %r delta (%r difference)' \
+              % (first, second, tolerance, difference)
+        raise AssertionError(msg)
+
     # ============================
     # tests for system-related API
     # ============================
@@ -251,26 +315,42 @@ class TestCase(unittest.TestCase):
         x = psutil.TOTAL_PHYMEM
         assert isinstance(x, (int, long))
         assert x > 0
+        self.assertEqual(x, psutil.virtual_memory().total)
 
     def test_BOOT_TIME(self):
         x = psutil.BOOT_TIME
         assert isinstance(x, float)
         assert x > 0
+        assert x < time.time(), x
 
     def test_NUM_CPUS(self):
         self.assertEqual(psutil.NUM_CPUS, len(psutil.cpu_times(percpu=True)))
+        assert psutil.NUM_CPUS >= 1, psutil.NUM_CPUS
+
+    @skipUnless(POSIX)
+    def test_PAGESIZE(self):
+        # pagesize is used internally to perform different calculations
+        # and it's determined by using SC_PAGE_SIZE; make sure
+        # getpagesize() returns the same value.
+        import resource
+        self.assertEqual(os.sysconf("SC_PAGE_SIZE"), resource.getpagesize())
 
     def test_deprecated_apis(self):
         warnings.filterwarnings("error")
         p = psutil.Process(os.getpid())
         try:
+            self.assertRaises(DeprecationWarning, psutil.virtmem_usage)
             self.assertRaises(DeprecationWarning, psutil.used_phymem)
             self.assertRaises(DeprecationWarning, psutil.avail_phymem)
             self.assertRaises(DeprecationWarning, psutil.total_virtmem)
             self.assertRaises(DeprecationWarning, psutil.used_virtmem)
             self.assertRaises(DeprecationWarning, psutil.avail_virtmem)
+            self.assertRaises(DeprecationWarning, psutil.phymem_usage)
             self.assertRaises(DeprecationWarning, psutil.get_process_list)
             self.assertRaises(DeprecationWarning, psutil.get_process_list)
+            if LINUX:
+                self.assertRaises(DeprecationWarning, psutil.phymem_buffers)
+                self.assertRaises(DeprecationWarning, psutil.cached_phymem)
             try:
                 p.nice
             except DeprecationWarning:
@@ -280,29 +360,43 @@ class TestCase(unittest.TestCase):
         finally:
             warnings.resetwarnings()
 
-    def test_phymem_usage(self):
-        mem = psutil.phymem_usage()
-        assert mem.total > 0, mem
-        assert mem.used > 0, mem
-        assert mem.free > 0, mem
-        assert 0 <= mem.percent <= 100, mem
+    def test_deprecated_apis_retval(self):
+        warnings.filterwarnings("ignore")
+        p = psutil.Process(os.getpid())
+        try:
+            self.assertEqual(psutil.total_virtmem(), psutil.swap_memory().total)
+            self.assertEqual(psutil.get_process_list(), list(psutil.process_iter()))
+            self.assertEqual(p.nice, p.get_nice())
+        finally:
+            warnings.resetwarnings()
 
-    def test_virtmem_usage(self):
-        mem = psutil.virtmem_usage()
+    def test_virtual_memory(self):
+        mem = psutil.virtual_memory()
+        assert mem.total > 0, mem
+        assert mem.available > 0, mem
+        assert 0 <= mem.percent <= 100, mem
+        assert mem.used > 0, mem
+        assert mem.free >= 0, mem
+        for name in mem._fields:
+            if name != 'total':
+                value = getattr(mem, name)
+                if not value >= 0:
+                    self.fail("%r < 0 (%s)" % (name, value))
+                if value > mem.total:
+                    self.fail("%r > total (total=%s, %s=%s)" \
+                              % (name, mem.total, name, value))
+
+    def test_swap_memory(self):
+        mem = psutil.swap_memory()
         assert mem.total > 0, mem
         assert mem.used >= 0, mem
         assert mem.free > 0, mem
         assert 0 <= mem.percent <= 100, mem
-
-    @skipUnless(LINUX)
-    def test_phymem_buffers(self):
-        x = psutil.phymem_buffers()
-        assert isinstance(x, (int, long))
-        assert x >= 0, x
+        assert mem.sin >= 0, mem
+        assert mem.sout >= 0, mem
 
     def test_pid_exists(self):
-        sproc = get_test_subprocess()
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(wait=True)
         assert psutil.pid_exists(sproc.pid)
         p = psutil.Process(sproc.pid)
         p.kill()
@@ -463,10 +557,18 @@ class TestCase(unittest.TestCase):
             self.assertEqual(nt[1], nt.bytes_recv)
             self.assertEqual(nt[2], nt.packets_sent)
             self.assertEqual(nt[3], nt.packets_recv)
+            self.assertEqual(nt[4], nt.errin)
+            self.assertEqual(nt[5], nt.errout)
+            self.assertEqual(nt[6], nt.dropin)
+            self.assertEqual(nt[7], nt.dropout)
             assert nt.bytes_sent >= 0, nt
             assert nt.bytes_recv >= 0, nt
             assert nt.packets_sent >= 0, nt
             assert nt.packets_recv >= 0, nt
+            assert nt.errin >= 0, nt
+            assert nt.errout >= 0, nt
+            assert nt.dropin >= 0, nt
+            assert nt.dropout >= 0, nt
 
         ret = psutil.network_io_counters(pernic=False)
         check_ntuple(ret)
@@ -513,9 +615,8 @@ class TestCase(unittest.TestCase):
     # ====================
 
     def test_kill(self):
-        sproc = get_test_subprocess()
+        sproc = get_test_subprocess(wait=True)
         test_pid = sproc.pid
-        wait_for_pid(test_pid)
         p = psutil.Process(test_pid)
         name = p.name
         p.kill()
@@ -523,9 +624,8 @@ class TestCase(unittest.TestCase):
         self.assertFalse(psutil.pid_exists(test_pid) and name == PYTHON)
 
     def test_terminate(self):
-        sproc = get_test_subprocess()
+        sproc = get_test_subprocess(wait=True)
         test_pid = sproc.pid
-        wait_for_pid(test_pid)
         p = psutil.Process(test_pid)
         name = p.name
         p.terminate()
@@ -675,9 +775,8 @@ class TestCase(unittest.TestCase):
             self.fail("expected: %s, found: %s" %(ktime, kernel_time))
 
     def test_create_time(self):
-        sproc = get_test_subprocess()
+        sproc = get_test_subprocess(wait=True)
         now = time.time()
-        wait_for_pid(sproc.pid)
         p = psutil.Process(sproc.pid)
         create_time = p.create_time
 
@@ -825,6 +924,9 @@ class TestCase(unittest.TestCase):
         assert percent2 > percent1
         del memarr
 
+#    def test_get_ext_memory_info(self):
+#       # tested later in fetch all test suite
+
     def test_get_memory_maps(self):
         p = psutil.Process(os.getpid())
         maps = p.get_memory_maps()
@@ -863,8 +965,7 @@ class TestCase(unittest.TestCase):
         self.assertEqual(psutil.Process(sproc.pid).pid, sproc.pid)
 
     def test_is_running(self):
-        sproc = get_test_subprocess()
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(wait=True)
         p = psutil.Process(sproc.pid)
         assert p.is_running()
         assert p.is_running()
@@ -874,10 +975,10 @@ class TestCase(unittest.TestCase):
         assert not p.is_running()
 
     def test_exe(self):
-        sproc = get_test_subprocess()
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(wait=True)
+        exe = psutil.Process(sproc.pid).exe
         try:
-            self.assertEqual(psutil.Process(sproc.pid).exe, PYTHON)
+            self.assertEqual(exe, PYTHON)
         except AssertionError:
             # certain platforms such as BSD are more accurate returning:
             # "/usr/local/bin/python2.7"
@@ -885,18 +986,15 @@ class TestCase(unittest.TestCase):
             # "/usr/local/bin/python"
             # We do not want to consider this difference in accuracy
             # an error.
-            name = psutil.Process(sproc.pid).exe
-            adjusted_name = PYTHON[:len(name)]
-            self.assertEqual(name, adjusted_name)
+            ver = "%s.%s" % (sys.version_info.major, sys.version_info.minor)
+            self.assertEqual(exe.replace(ver, ''), PYTHON.replace(ver, ''))
 
     def test_cmdline(self):
-        sproc = get_test_subprocess([PYTHON, "-E"])
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess([PYTHON, "-E"], wait=True)
         self.assertEqual(psutil.Process(sproc.pid).cmdline, [PYTHON, "-E"])
 
     def test_name(self):
-        sproc = get_test_subprocess(PYTHON)
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(PYTHON, wait=True)
         pyexe = os.path.basename(os.path.realpath(sys.executable)).lower()
         self.assertEqual(psutil.Process(sproc.pid).name.lower(), pyexe)
 
@@ -995,20 +1093,16 @@ class TestCase(unittest.TestCase):
 
     @skipIf(not hasattr(psutil.Process, "getcwd"))
     def test_getcwd(self):
-        sproc = get_test_subprocess()
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(wait=True)
         p = psutil.Process(sproc.pid)
         self.assertEqual(p.getcwd(), os.getcwd())
 
     @skipIf(not hasattr(psutil.Process, "getcwd"))
     def test_getcwd_2(self):
         cmd = [PYTHON, "-c", "import os, time; os.chdir('..'); time.sleep(10)"]
-        sproc = get_test_subprocess(cmd)
-        wait_for_pid(sproc.pid)
+        sproc = get_test_subprocess(cmd, wait=True)
         p = psutil.Process(sproc.pid)
-        time.sleep(0.1)
-        expected_dir = os.path.dirname(os.getcwd())
-        self.assertEqual(p.getcwd(), expected_dir)
+        call_until(p.getcwd, "ret == os.path.dirname(os.getcwd())", timeout=1)
 
     @skipIf(not hasattr(psutil.Process, "get_cpu_affinity"))
     def test_cpu_affinity(self):
@@ -1032,8 +1126,8 @@ class TestCase(unittest.TestCase):
         p = psutil.Process(os.getpid())
         files = p.get_open_files()
         self.assertFalse(TESTFN in files)
-        f = open(TESTFN, 'r')
-        time.sleep(.1)
+        f = open(TESTFN, 'w')
+        call_until(p.get_open_files, "len(ret) != %i" % len(files))
         filenames = [x.path for x in p.get_open_files()]
         self.assertIn(TESTFN, filenames)
         f.close()
@@ -1042,9 +1136,7 @@ class TestCase(unittest.TestCase):
 
         # another process
         cmdline = "import time; f = open(r'%s', 'r'); time.sleep(100);" % TESTFN
-        sproc = get_test_subprocess([PYTHON, "-c", cmdline])
-        wait_for_pid(sproc.pid)
-        time.sleep(0.1)
+        sproc = get_test_subprocess([PYTHON, "-c", cmdline], wait=True)
         p = psutil.Process(sproc.pid)
         for x in range(100):
             filenames = [x.path for x in p.get_open_files()]
@@ -1058,7 +1150,7 @@ class TestCase(unittest.TestCase):
 
     def test_get_open_files2(self):
         # test fd and path fields
-        fileobj = open(TESTFN, 'r')
+        fileobj = open(TESTFN, 'w')
         p = psutil.Process(os.getpid())
         for path, fd in p.get_open_files():
             if path == fileobj.name or fd == fileobj.fileno():
@@ -1124,6 +1216,28 @@ class TestCase(unittest.TestCase):
         self.assertEqual(len(cons), 1)
         self.assertEqual(cons[0].local_address[0], '::1')
 
+    @skipUnless(hasattr(socket, 'AF_UNIX'))
+    def test_get_connections_unix(self):
+        # tcp
+        safe_remove(TESTFN)
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        sock.bind(TESTFN)
+        conn = psutil.Process(os.getpid()).get_connections(kind='unix')[0]
+        self.assertEqual(conn.fd, sock.fileno())
+        self.assertEqual(conn.family, socket.AF_UNIX)
+        self.assertEqual(conn.type, socket.SOCK_STREAM)
+        self.assertEqual(conn.local_address, TESTFN)
+        self.assertTrue(not conn.remote_address)
+        self.assertTrue(not conn.status)
+        sock.close()
+        # udp
+        safe_remove(TESTFN)
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+        sock.bind(TESTFN)
+        conn = psutil.Process(os.getpid()).get_connections(kind='unix')[0]
+        self.assertEqual(conn.type, socket.SOCK_DGRAM)
+        sock.close()
+
     @skipUnless(hasattr(socket, "fromfd") and not WINDOWS)
     def test_connection_fromfd(self):
         sock = socket.socket()
@@ -1239,7 +1353,7 @@ class TestCase(unittest.TestCase):
     def test_get_num_fds(self):
         p = psutil.Process(os.getpid())
         start = p.get_num_fds()
-        file = open(TESTFN, 'r')
+        file = open(TESTFN, 'w')
         self.assertEqual(p.get_num_fds(), start + 1)
         sock = socket.socket()
         self.assertEqual(p.get_num_fds(), start + 2)
@@ -1247,6 +1361,15 @@ class TestCase(unittest.TestCase):
         sock.close()
         self.assertEqual(p.get_num_fds(), start)
 
+    def test_get_num_ctx_switches(self):
+        p = psutil.Process(os.getpid())
+        before = sum(p.get_num_ctx_switches())
+        for x in range(50000):
+            after = sum(p.get_num_ctx_switches())
+            if after > before:
+                return
+        self.fail("num ctx switches still the same after 50.000 iterations")
+
     def test_parent_ppid(self):
         this_parent = os.getpid()
         sproc = get_test_subprocess()
@@ -1312,11 +1435,13 @@ class TestCase(unittest.TestCase):
             self.assertEqual(len(c), len(set(c)))
 
     def test_suspend_resume(self):
-        sproc = get_test_subprocess()
+        sproc = get_test_subprocess(wait=True)
         p = psutil.Process(sproc.pid)
         p.suspend()
-        time.sleep(0.1)
-        self.assertEqual(p.status, psutil.STATUS_STOPPED)
+        for x in range(100):
+            if p.status == psutil.STATUS_STOPPED:
+                break
+            time.sleep(0.01)
         self.assertEqual(str(p.status), "stopped")
         p.resume()
         assert p.status != psutil.STATUS_STOPPED, p.status
@@ -1425,11 +1550,10 @@ class TestCase(unittest.TestCase):
         except psutil.AccessDenied:
             pass
 
-        if OSX : #and os.geteuid() != 0:
-            self.assertRaises(psutil.AccessDenied, p.get_memory_info)
-            self.assertRaises(psutil.AccessDenied, p.get_cpu_times)
-        else:
+        try:
             p.get_memory_info()
+        except psutil.AccessDenied:
+            pass
 
         # username property
         if POSIX:
@@ -1476,9 +1600,6 @@ class TestFetchAllProcesses(unittest.TestCase):
         excluded_names = ['send_signal', 'suspend', 'resume', 'terminate',
                           'kill', 'wait', 'as_dict', 'get_cpu_percent', 'nice',
                           'parent', 'get_children', 'pid']
-        # XXX - skip slow lsof implementation;
-        if BSD:
-           excluded_names += ['get_connections']
         attrs = []
         for name in dir(psutil.Process):
             if name.startswith("_"):
@@ -1593,10 +1714,27 @@ class TestFetchAllProcesses(unittest.TestCase):
         self.assertTrue(ret.rss >= 0)
         self.assertTrue(ret.vms >= 0)
 
+    def get_ext_memory_info(self, ret):
+        for name in ret._fields:
+            self.assertTrue(getattr(ret, name) >= 0)
+        if POSIX and ret.vms != 0:
+            # VMS is always supposed to be the highest
+            for name in ret._fields:
+                if name != 'vms':
+                    value = getattr(ret, name)
+                    assert ret.vms > value, ret
+        elif WINDOWS:
+            assert ret.peak_wset >= ret.wset, ret
+            assert ret.peak_paged_pool >= ret.paged_pool, ret
+            assert ret.peak_nonpaged_pool >= ret.nonpaged_pool, ret
+            assert ret.peak_pagefile >= ret.pagefile, ret
+
     def get_open_files(self, ret):
         for f in ret:
-            if f.fd != -1:  # windows
-                assert f.fd >= 0, f
+            if WINDOWS:
+                assert f.fd == -1, f
+            else:
+                assert isinstance(f.fd, int), f
             assert os.path.isabs(f.path), f
             assert os.path.isfile(f.path), f
 
@@ -1686,7 +1824,10 @@ class TestFetchAllProcesses(unittest.TestCase):
                     assert value >= 0, value
 
     def get_num_handles(self, ret):
-        assert ret > 0
+        if WINDOWS:
+            assert ret >= 0
+        else:
+            assert ret > 0
 
     def get_nice(self, ret):
         if POSIX:
@@ -1696,6 +1837,9 @@ class TestFetchAllProcesses(unittest.TestCase):
                           if x.endswith('_PRIORITY_CLASS')]
             self.assertIn(ret, priorities)
 
+    def get_num_ctx_switches(self, ret):
+        self.assertTrue(ret.voluntary >= 0)
+        self.assertTrue(ret.involuntary >= 0)
 
 if hasattr(os, 'getuid'):
     class LimitedUserTestCase(TestCase):
@@ -1739,6 +1883,13 @@ if hasattr(os, 'getuid'):
                 self.fail("exception not raised")
 
 
+def cleanup():
+    reap_children(search_all=True)
+    DEVNULL.close()
+    safe_remove(TESTFN)
+
+atexit.register(cleanup)
+
 def test_main():
     tests = []
     test_suite = unittest.TestSuite()
@@ -1754,6 +1905,8 @@ def test_main():
         from _linux import LinuxSpecificTestCase as stc
     elif WINDOWS:
         from _windows import WindowsSpecificTestCase as stc
+        from _windows import TestDualProcessImplementation
+        tests.append(TestDualProcessImplementation)
     elif OSX:
         from _osx import OSXSpecificTestCase as stc
     elif BSD:
@@ -1764,18 +1917,12 @@ def test_main():
         if os.getuid() == 0:
             tests.append(LimitedUserTestCase)
         else:
-            atexit.register(warnings.warn, "Couldn't run limited user tests ("
-                         "super-user privileges are required)", RuntimeWarning)
+            atexit.register(warn, "Couldn't run limited user tests ("
+                                  "super-user privileges are required)")
 
     for test_class in tests:
         test_suite.addTest(unittest.makeSuite(test_class))
-
-    f = open(TESTFN, 'w')
-    f.close()
-    atexit.register(lambda: os.remove(TESTFN))
-
     unittest.TextTestRunner(verbosity=2).run(test_suite)
-    DEVNULL.close()
 
 if __name__ == '__main__':
     test_main()
